///	\file			MFC3045Device.h
///	\date			2022
///	\copyright (c)	Metrolab Technology S.A., Geneva, Switzerland (www.metrolab.com)
///	See the included file License Terms.pdf for the licensing conditions.

#pragma once

// Standard includes
#include <string>

// Personal includes
#include "VISAInstrument.h"
#include "MFC3045.h"
#include "MFCDevice.h"
#include "InstrumentCatalog.h"
#include "MFCController.h"
#include "Helpers.h"

using namespace MTL;
using namespace MTL::Instrument;

namespace MTL {

	//--------------------------------------------------------------------------//
	//								MFC3045 Device
	//--------------------------------------------------------------------------//

	class CMFC3045Device: public CMFCDevice
	{
		//--------------------------------------------------------------------------//
		//								Attributes
		//--------------------------------------------------------------------------//
	private:
		CMFCController &							m_rMFCController;
		CMFC3045Instrument							m_MFC3045;
		sSerialPortSettings							m_SerialParms;
		std::string									m_PaSerialNum;
		U8											m_NbProbesPA;
		CInstrumentCatalog::sInstrumentState		m_LastConfigured;
		U64											m_MaxMeasTimeOutMs;
		sStatusAllReg								m_AllRegStatus;

		//constants:
			const int			SEARCH_POLLING_PERIOD_MS		= 100;
			const int			MEAS_POLLING_PERIOD_MS			= 100;

		//--------------------------------------------------------------------------//
		//							Constructor / Destructor
		//--------------------------------------------------------------------------//
	public:
		CMFC3045Device (	CMFCController &	rMFCController, CVISAResourceManager & rRM,
							tResourceName Rsrc, CInstrumentCatalog::sMFC3045SerialPortSettings & rSerialSetting);
		virtual ~CMFC3045Device ();

		//--------------------------------------------------------------------------//
		//								Utilities
		//--------------------------------------------------------------------------//
	private:
		F32 l_ConvertNormdHzToMHz (const I32 & rFreqdHz);
		I32 l_ConvertNormMHzTodHz (const F32 & rFreqMHz);

		bool l_UtcToStrDateISO8601(const std::time_t & rUtcDate, std::string & rStrDate);
		bool l_ParseSerialSettings (	const CInstrumentCatalog::sMFC3045SerialPortSettings & rSerialSetting,
										sSerialPortSettings & rVisaSerialSetting);
		bool l_SerialCommunicationSuccess();
		bool l_CheckPAConnected(bool & rPAPresent);
		bool l_ConfigurationChanged (CInstrumentCatalog::sInstrumentState lastcfg, CInstrumentCatalog::sInstrumentState newcfg);
		bool l_ConfigureForOperation (const CInstrumentCatalog::sInstrumentState & rInstrState, sMFCDeviceError & rError);
		bool l_ComputeMaxMeasTimeout(U64 & rMaxTimeOutMs);
		void l_ComputeAveraging (const std::vector<U32> & rFluxListdHz, F64 & rReturnedAveragingMHz, F64 & rReturnedStdDevppm);
		bool l_WriteNormalizationRAM(const CInstrumentCatalog::tNormalizationTable & rNewNormalizationTable, sMFCDeviceError & rError);

	public:
		bool AbortInstrument ();
		bool IsInstrumentConnected();

		//--------------------------------------------------------------------------//
		//							Available Operations
		//--------------------------------------------------------------------------//
	public:
		sMFCDeviceError Connect (CInstrumentCatalog::sInstrumentInformation & rInstrInfo);
		sMFCDeviceError ScanPA (	CInstrumentCatalog::sProbeArrayInformation & rPAInfo,
									CInstrumentCatalog::sInstrumentConfiguration & rInstrConfig, bool &rPaInfoChanged);
		sMFCDeviceError Search (const CInstrumentCatalog::sInstrumentState & rInstrState);
		sMFCDeviceError Measure (const CInstrumentCatalog::sInstrumentState & rInstrState);
		sMFCDeviceError NewNormalizationRequest (	const CInstrumentCatalog::sNormalization & rNewNormalizationRequest,
													CInstrumentCatalog::sNormalization & rReturnedNormalization);
		sMFCDeviceError GetNormalizationDate(CInstrumentCatalog::tISO8601DateTime & rNormDate);
		sMFCDeviceError SetRBBusyLedState (const CInstrumentCatalog::eRemoteBoxBusyLedState & rRBBusyLedState);
		sMFCDeviceError RemoteBoxButtonStatus (bool & rRemoteStartPressed);
		sMFCDeviceError ChangeSerialParmsOnConnectedInstr(const CInstrumentCatalog::sMFC3045SerialPortSettings & rNewSerialSetting);
	}; // class
} // namespace MTL
