///	\file			USBRemoteBoxController.h
///	\date			2020
///	\copyright (c)	Metrolab Technology S.A., Geneva, Switzerland (www.metrolab.com)
///	See the included file License Terms.pdf for the licensing conditions.

#pragma once

// Personal includes
#include "USBRemoteBox.h"
#include "Exception.h"
#include "Helpers.h"
#include "Synchronization.h"
#include "ThreadedMailbox.h"
#include "MFCManagerCatalog.h"

using namespace MTL;
using namespace MTL::Instrument;
using namespace MTL::ThreadedMailbox;

namespace MTL {

	//--------------------------------------------------------------------------//
	//								Remote Box 
	//--------------------------------------------------------------------------//
	struct sUSBRemoteBoxControllerMailBoxMsg {
		enum eMsgType { kTimerEvent, kCatalogNotification }		MsgType;
		CAsyncNotification *									pCatNot;
		sUSBRemoteBoxControllerMailBoxMsg()
			: pCatNot(nullptr) {}
		sUSBRemoteBoxControllerMailBoxMsg(eMsgType MT, CAsyncNotification * pNot)
			: MsgType(MT), pCatNot(pNot) {}
	};
	class CUSBRemoteBoxController : private CThreadedMailBox<sUSBRemoteBoxControllerMailBoxMsg>
	{
	//----------------------------------------------------------------------//
	// Constants
	private:
		static const std::string	RSRC_FILTER;
		static const int			REFRESH_RATE_MS;

	//----------------------------------------------------------------------//
	// Attributes
	private:
		CVISAResourceManager &			m_rRsrcMan;
		CUSBRemoteBoxInterface			m_RBCat;
		CAsyncSnaphsotNotifier			m_ASManagerNotifier;
		CUSBRemoteBox *					m_pRemoteBox;
		CTimer							m_Timer;
		CUSBRemoteBox::uLEDs			m_LEDs;
		CUSBRemoteBox::uButtons			m_Buttons;

	//----------------------------------------------------------------------//
	// Methods
	public:
		//----------------------------------------------------------------------//
		// Constructors / Destructors
		CUSBRemoteBoxController(CVISAResourceManager & rRM, CMFCManagerCatalog & rCatInstance);
		virtual ~CUSBRemoteBoxController();

		//----------------------------------------------------------------------//
		// Start and stop control
		void Operate();
		void Terminate();

	// Lives in the timer thread
	private:
		void l_PostTimerEvent();										// Called when timer elapses. Adds timer event to queue.
	// Lives in the Manager thread
	private:
		void l_PostManagerNotification(CAsyncNotification * pASNot);	// Called when a subscribed observer is touched. Adds notification to queue.

	private:
		//----------------------------------------------------------------------//
		// Internal
		void l_ThreadTask(CThreadedMailBox::eEventType EvtType, sUSBRemoteBoxControllerMailBoxMsg * pMsg);
		void l_AsyncManagerNotificationHandler(CAsyncNotification * pASNot);
		void l_Refresh();
		void l_SetBusyState(bool Busy);
	};

} // namespace MTL
