#pragma once

// Standard includes
#include <string>

// Personal includes
#include "VISAInstrument.h"

namespace MTL {
	namespace Instrument {

	//----------------------------------------------------------------------//
	//						USB Remote Box Interrface						//
	//----------------------------------------------------------------------//
	class CUSBRemoteBox : private CVISAInstrument
	{
	public:
		static const std::string		RSRC_FILTER;
		union uLEDs {
			struct sLEDs
			{
				uint32_t ON : 1;
				uint32_t BUSY : 1;
			}	LEDs;
			uint32_t	Buf;
		};
		union uButtons
		{
			struct sButtons
			{
				uint32_t StartState : 1;
				uint32_t StartEvent : 1;
				uint32_t ResetState : 1;
				uint32_t ResetEvent : 1;
			}	BUTs;
			uint32_t	Buf;
		};
	private:
		const uint32_t		m_Timeout_ms = 2000;
		enum usb_request_type {
			USB_REQUEST_TYPE_STANDARD = (0x00 << 5),
			USB_REQUEST_TYPE_CLASS = (0x01 << 5),
			USB_REQUEST_TYPE_VENDOR = (0x02 << 5),
			USB_REQUEST_TYPE_RESERVED = (0x03 << 5)
		};
		enum usb_request_recipient {
			USB_RECIPIENT_DEVICE = 0x00,
			USB_RECIPIENT_INTERFACE = 0x01,
			USB_RECIPIENT_ENDPOINT = 0x02,
			USB_RECIPIENT_OTHER = 0x03,
		};
		enum usb_endpoint_direction {
			USB_ENDPOINT_IN = 0x80,	// Device to host
			USB_ENDPOINT_OUT = 0x00	// Host to device
		};

	public:
		//----------------------------------------------------------------------//
		// Constructors / destructors
		CUSBRemoteBox(CVISAResourceManager & rRM, tResourceName Rsrc);
		virtual ~CUSBRemoteBox();

		//----------------------------------------------------------------------//
		// Open / Close
		bool Connect();
		void Disconnect();

		//----------------------------------------------------------------------//
		// Read / Write
		bool Update(const uLEDs LEDs, uButtons & rButtons);
	};

	}
}
